# -*- coding: utf-8 -*-
# pylint: disable=chained-comparison,invalid-name,line-too-long,protected-access,too-many-arguments,too-many-boolean-expressions,too-many-branches,too-many-lines,too-many-locals,too-many-statements,unneeded-not,unused-import
u"""
Interfaces for the NAG Mark 30.1 `machine` Chapter.

``machine`` - Machine Constants

This module is concerned with **parameters** which characterise certain
aspects of the **computing environment** in which the NAG Library is
implemented.
They relate primarily to floating-point arithmetic, but also to integer
arithmetic, the elementary functions and exception handling.
The values of the parameters vary from one implementation of the NAG
Library to another, but within the context of a single implementation
they are constants.

The parameters are intended for use primarily by other functions in the
NAG Library, but users of the NAG Library may sometimes need to refer to
them directly.

For full information please refer to the NAG Library document

https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02intro.html
"""

# NAG Copyright 2017-2024.

from ..base import machine as _b_machine

def sinarg_max():
    u"""
    The largest permissible argument for sin and cos.

    ``sinarg_max`` returns the largest positive argument for which the
    Fortran intrinsic functions SIN and COS return a result with some
    meaningful accuracy.

    For full information please refer to the NAG Library document for
    x02ah

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02ahf.html

    Returns
    -------
    maxsc : float
        The largest positive argument for which the Fortran intrinsic
        functions SIN and COS return a result with some meaningful
        accuracy.

    Notes
    -----
    The trigonometric functions sin and cos supplied in some compiler
    run-time libraries do not return accurate results when their
    argument is large.
    Often the related accuracy of a result gets progressively worse as
    the argument gets larger. ``sinarg_max`` gives a value beyond which
    the compiler run-time library returns results with no relative
    accuracy at all.
    Note that some run-time libraries do return accurate results for all
    arguments to sin and cos.
    """

    _return_scalar = _b_machine.sinarg_max(
    )
    return _return_scalar

def precision():
    u"""
    The machine precision.

    ``precision`` returns epsilon, the value machine precision.

    For full information please refer to the NAG Library document for
    x02aj

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02ajf.html

    Returns
    -------
    eps : float
        epsilon, the value machine precision.

    Notes
    -----
    ``precision`` returns machine precision, computed as epsilon =
    1/2*b^(1-p), where b is the arithmetic base (see ``model_base``) and
    p is the number of significant base-b digits (see ``model_digits``).

    It is important to note that the definition of epsilon here differs
    from that in ISO Fortran 95 (1997).

    References
    ----------
    ISO Fortran 95, 1997, `ISO Fortran 95 programming language`,
    (ISO/IEC 1539--1:1997)
    """

    _return_scalar = _b_machine.precision(
    )
    return _return_scalar

def real_smallest():
    u"""
    The smallest positive model number.

    ``real_smallest`` returns the smallest positive floating-point
    number.

    For full information please refer to the NAG Library document for
    x02ak

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02akf.html

    Returns
    -------
    tiny_nag : float
        The smallest positive floating-point number.

    Notes
    -----
    ``real_smallest`` returns the smallest positive number in the model
    of floating-point arithmetic described in [the X02 Introduction].
    The returned value is equal to b^(e_min-1), where b is the
    arithmetic base (see ``model_base``) and e_min is the minimum
    exponent (see ``model_minexp``) in the model.
    """

    _return_scalar = _b_machine.real_smallest(
    )
    return _return_scalar

def real_largest():
    u"""
    The largest positive model number.

    ``real_largest`` returns the largest positive floating-point number.

    For full information please refer to the NAG Library document for
    x02al

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02alf.html

    Returns
    -------
    huge_nag : float
        The largest positive floating-point number.

    Notes
    -----
    ``real_largest`` returns the largest positive number in the model of
    floating-point arithmetic described in [the X02 Introduction].
    The returned value is equal to (1-b^(-p))*b^(e_max), where b is the
    arithmetic base (see ``model_base``) and e_max is the maximum
    exponent (see ``model_maxexp``) in the model.
    """

    _return_scalar = _b_machine.real_largest(
    )
    return _return_scalar

def real_safe():
    u"""
    The safe range parameter.

    ``real_safe`` returns the **safe range** of floating-point
    arithmetic.

    For full information please refer to the NAG Library document for
    x02am

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02amf.html

    Returns
    -------
    saferange : float
        The **safe range** of floating-point arithmetic.

    Notes
    -----
    ``real_safe`` is defined to be the smallest positive model number z
    such that for any x in the range [z,1/z] the following can be
    computed without undue loss of accuracy, overflow, underflow or
    other error:

        -x

        1/x

        -1/x

        sqrt(x)

        log(x)

        exp(log(x))

        y^(log(x)/log(y)) for any y
    """

    _return_scalar = _b_machine.real_safe(
    )
    return _return_scalar

def complex_safe():
    u"""
    The safe range parameter for complex floating-point arithmetic.

    ``complex_safe`` returns the **safe range** of complex
    floating-point arithmetic.

    For full information please refer to the NAG Library document for
    x02an

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02anf.html

    Returns
    -------
    zrange : float
        The **safe range** of complex floating-point arithmetic.

    Notes
    -----
    ``complex_safe`` is defined to be the smallest positive model number
    z such that for any x in the range [z,1/z] the following can be
    computed without undue loss of accuracy, overflow, underflow or
    other error:

        -w

        1/w

        -1/w

        sqrt(w)

        log(w)

        exp(log(w))

        y^(log(w)/log(y)) for any y

        |w|

    where w is any of x, ix, x+ix, 1/x, i/x, 1/x+i/x, and i is the
    square root of -1.
    """

    _return_scalar = _b_machine.complex_safe(
    )
    return _return_scalar

def integer_max():
    u"""
    The largest representable integer.

    ``integer_max`` returns the largest representable positive integer
    value.

    For full information please refer to the NAG Library document for
    x02bb

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02bbf.html

    Returns
    -------
    hugeint : int
        The largest representable positive integer value.
    """

    _return_scalar = _b_machine.integer_max(
    )
    return _return_scalar

def decimal_digits():
    u"""
    The maximum number of decimal digits that can be represented.

    ``decimal_digits`` returns the maximum number of decimal digits
    which can be accurately represented over the whole range of
    floating-point numbers.

    For full information please refer to the NAG Library document for
    x02be

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02bef.html

    Returns
    -------
    iprec : int
        The maximum number of decimal digits which can be accurately
        represented over the whole range of floating-point numbers.
    """

    _return_scalar = _b_machine.decimal_digits(
    )
    return _return_scalar

def model_base():
    u"""
    The floating-point model parameter, b.

    ``model_base`` returns the base argument b of the model of
    floating-point arithmetic as described in [the X02 Introduction].

    For full information please refer to the NAG Library document for
    x02bh

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02bhf.html

    Returns
    -------
    irad : int
        The base argument b of the model of floating-point arithmetic as
        described in [the X02 Introduction].
    """

    _return_scalar = _b_machine.model_base(
    )
    return _return_scalar

def model_digits():
    u"""
    The floating-point model parameter, p.

    ``model_digits`` returns the precision argument p (i.e., the number
    of significant base-b digits) of the model of floating-point
    arithmetic described in [the X02 Introduction].

    For full information please refer to the NAG Library document for
    x02bj

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02bjf.html

    Returns
    -------
    idigs : int
        The precision argument p (i.e., the number of significant base-b
        digits) of the model of floating-point arithmetic described in
        [the X02 Introduction].
    """

    _return_scalar = _b_machine.model_digits(
    )
    return _return_scalar

def model_minexp():
    u"""
    The floating-point model parameter e_min.

    ``model_minexp`` returns the minimum exponent argument e_min of the
    model of floating-point arithmetic described in [the X02
    Introduction].

    For full information please refer to the NAG Library document for
    x02bk

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02bkf.html

    Returns
    -------
    minexp : int
        The minimum exponent argument e_min of the model of
        floating-point arithmetic described in [the X02 Introduction].
    """

    _return_scalar = _b_machine.model_minexp(
    )
    return _return_scalar

def model_maxexp():
    u"""
    The floating-point model parameter e_max.

    ``model_maxexp`` returns the maximum exponent argument e_max of the
    model of floating-point arithmetic described in [the X02
    Introduction].

    For full information please refer to the NAG Library document for
    x02bl

    https://support.nag.com/numeric/nl/nagdoc_30.1/flhtml/x02/x02blf.html

    Returns
    -------
    maxexp : int
        The maximum exponent argument e_max of the model of
        floating-point arithmetic described in [the X02 Introduction].
    """

    _return_scalar = _b_machine.model_maxexp(
    )
    return _return_scalar
